<?php

namespace Drupal\video_embed_html5\Plugin\video_embed_field\Provider;

use Drupal\Core\Extension\ModuleHandlerInterface;
use Drupal\video_embed_field\ProviderPluginBase;
use FFMpeg\Coordinate\TimeCode;
use FFMpeg\FFMpeg;
use GuzzleHttp\ClientInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * A Html5 provider plugin.
 *
 * @VideoEmbedProvider(
 *   id = "html_5",
 *   title = @Translation("HTML5")
 * )
 */
class Html5 extends ProviderPluginBase {

  /** @var FFMpeg $phpFFMpeg */
  protected $phpFFMpeg;
  protected $filename;

  /**
   * Html5 constructor.
   * @param string $configuration
   * @param string $plugin_id
   * @param array $plugin_definition
   * @param \GuzzleHttp\ClientInterface $http_client
   * @param \Drupal\Core\Extension\ModuleHandlerInterface $module_handler
   */
  public function __construct($configuration, $plugin_id, array $plugin_definition,
                              \GuzzleHttp\ClientInterface $http_client, ModuleHandlerInterface $module_handler) {
    parent::__construct($configuration, $plugin_id, $plugin_definition, $http_client);

    $this->phpFFMpeg = FALSE;
    if ($module_handler->moduleExists('php_ffmpeg')) {
      $this->phpFFMpeg = \Drupal::service('php_ffmpeg');
    }

    // Set filename for thumbnail.
    list($video_url, $video_type) = $this->getVideoId();
    $this->filename = md5($video_url);
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('http_client'),
      $container->get('module_handler')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function renderEmbedCode($width, $height, $autoplay) {
    list($video_url, $video_type) = $this->getVideoId();
    return [
      '#theme' => 'video_embed_html5',
      '#src' => $video_url,
      '#type' => 'video/' . $video_type,
      '#autoplay' => $autoplay,
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function downloadThumbnail() {
    if ($this->phpFFMpeg) {
      $local_uri = $this->getLocalThumbnailUri();
      if (!file_exists($local_uri)) {
        file_prepare_directory($this->thumbsDirectory, FILE_CREATE_DIRECTORY);
        try {
          list($video_url, $video_type) = $this->getVideoId();
          // Thumb does not exist yet. Generate it.
          $video = $this->phpFFMpeg->open($video_url);
          $video->frame(TimeCode::fromSeconds(1))
            ->save(drupal_realpath($this->thumbsDirectory) . '/' . $this->filename . '.jpg');
        } catch (\Exception $e) {
        }
      }
    }
    else {
      // Show info that php_ffmpeg is required to generate thumb.
      drupal_set_message($this->t('To use the "Thumbnail" formatter you need to install the <a href="@link" target="_blank">php_ffmpeg module</a>', [
        '@link' => 'https://www.drupal.org/project/php_ffmpeg',
      ]), 'error');
    }
  }

  /**
   * {@inheritdoc}
   */
  public function getLocalThumbnailUri() {
    return $this->thumbsDirectory . '/' . $this->filename . '.jpg';
  }

  /**
   * {@inheritdoc}
   */
  public function getRemoteThumbnailUrl() {
    // Don't need this as we override "downloadThumbnail".
    return FALSE;
  }

  /**
   * {@inheritdoc}
   */
  public static function getIdFromInput($input) {
    $pattern = "/\b(http|https):\/\/.*(mp4|ogg|webm)\b/";
    $matches = array();
    preg_match($pattern, $input, $matches);

    // Make sure there are values.
    if ($matches && isset($matches[2])) {
      return array($input, $matches[2]);
    }

    return FALSE;
  }
}
